@@ -1,6 +1,6 @@
 /*
     upad - A program for debugging, and uploading code to embedded devices.
-    Copyright (C) 2016 John Darrington
+    Copyright (C) 2016, 2019 John Darrington
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,8 +44,14 @@ enum cmd
     CMD_MD5_CHECK,
     CMD_ERASE_EEPROM_SECTOR,
     CMD_ERASE_PFLASH_SECTOR,
+    CMD_AUTOCONNECT,
     n_CMDS
   };
 
 #endif
 
+
+/* Local Variables:  */
+/* mode: c           */
+/* c-style: "gnu"    */
+/* End:              */
