@@ -1,6 +1,6 @@
 /*
     upad - A program for debugging, and uploading code to embedded devices.
-    Copyright (C) 2016 John Darrington
+    Copyright (C) 2016, 2019 John Darrington
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 
 #include "periph/gpio.h"
 #include "board.h"
+#include "xtimer.h"
 
 #include "sep.h"
 
@@ -31,9 +32,15 @@ int sep_start (int dev);
 int
 main (void)
 {
+  xtimer_init ();
   gpio_init (LED0_PIN, GPIO_OUT);
 
   sep_start (uart_dev);
 
   return 0;
 }
+
+/* Local Variables:  */
+/* mode: c           */
+/* c-style: "gnu"    */
+/* End:              */
