@@ -1,6 +1,6 @@
 /*
   upad - A program for debugging, and uploading code to embedded devices.
-  Copyright (C) 2016 John Darrington
+  Copyright (C) 2016, 2019 John Darrington
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -23,6 +23,11 @@
 
 #include "uploader.h"
 
-struct upload_format * create_binary_uploader (FILE *fp, uint32_t load_address);
+struct upload_format * create_binary_uploader (FILE *fp, tgt_addr load_address);
 
 #endif
+
+/* Local Variables:  */
+/* mode: c           */
+/* c-style: "gnu"    */
+/* End:              */
