@@ -1,6 +1,6 @@
 /*
   upad - A program for debugging, and uploading code to embedded devices.
-  Copyright (C) 2016 John Darrington
+  Copyright (C) 2016, 2019 John Darrington
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 
 #include <stdlib.h>
 #include <stdbool.h>
+#include <stdint.h>
 
 static inline  void *
 safe_realloc (void *ptr, size_t size)
@@ -40,16 +41,27 @@ enum msglevel
     MSG_DIAGNOSTIC
   };
 
+void upad_msg (enum msglevel, const char *fmt, ...)
+  __attribute__ ((format (printf, 2, 3)));
+void upad_errno (enum msglevel lvl, const char *fmt, ...)
+  __attribute__ ((format (printf, 2, 3)));
 
-void upad_msg (enum msglevel, const char *fmt, ...);
-void upad_errno (enum msglevel lvl, const char *fmt, ...);
-
-bool process_command (const char *cmd, size_t cmd_length, int fpr);
+bool process_command (const char *cmd, int fpr);
 
 struct bounce_code_usage;
 struct upload_format;
 int upload_format (int des, struct upload_format *uf, const struct bounce_code_usage
 		   *bcu);
 
+
+struct user_options;
+void canonicalise_control_file (struct user_options *opts);
+
+
 #endif
 
+
+/* Local Variables:  */
+/* mode: c           */
+/* c-style: "gnu"    */
+/* End:              */
