@@ -1,6 +1,6 @@
 /*
   upad - A program for debugging, and uploading code to embedded devices.
-  Copyright (C) 2016 John Darrington
+  Copyright (C) 2016, 2019 John Darrington
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #define TARGET_H
 
 #include <stdint.h>
+#include "upad.h"
 
 enum memclass
 {
@@ -42,7 +43,7 @@ struct memsector
     /* The minimum number of bytes which can be written in one operation */
     int write_sector;
 
-    uint32_t address;
+    tgt_addr address;
     uint32_t size;
 };
 
@@ -50,4 +51,11 @@ extern const struct memsector memmap[];
 
 extern const int n_memsectors;
 
+extern const tgt_addr RESET_VECTOR;
+
 #endif
+
+/* Local Variables:  */
+/* mode: c           */
+/* c-style: "gnu"    */
+/* End:              */
