@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2016 John Darrington
+  Copyright (C) 2016, 2019 John Darrington
 
   Copying and distribution of this file, with or without modification,
   are permitted in any medium without royalty provided the copyright
@@ -8,9 +8,6 @@
 */
 #include <stdint.h>
 
-#define __IOM volatile
-#define __IM volatile const
-#define __OM volatile
 
 typedef struct
 {
