@@ -1,6 +1,6 @@
 /*
   upad - A program for debugging, and uploading code to embedded devices.
-  Copyright (C) 2016 John Darrington
+  Copyright (C) 2016, 2019 John Darrington
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,9 +22,12 @@
 
 const struct memsector memmap[] =
 {
-  {MEMCLASS_RAM,    CMD_WRITE_MEM, 1,   1, 0x1000,   8192},
+  {MEMCLASS_OTHER,  CMD_WRITE_MEM, 0,   1, 0,        4095},
+  {MEMCLASS_RAM,    CMD_WRITE_MEM, 0,   1, 0x1000,   8192},
   {MEMCLASS_EEPROM, CMD_EEPROM,    4,   1, 0x100000, 2048},
   {MEMCLASS_PFLASH, CMD_PFLASH,    512, 8, 0xFE0000, 128 * 1024}
 };
 
-const int n_memsectors = 3;
+const int n_memsectors = 4;
+
+const tgt_addr RESET_VECTOR = 0xFFFFFC;
