@@ -1,6 +1,6 @@
 #!/bin/sh
 # upad - A program for debugging, and uploading code to embedded devices.
-# Copyright (C) 2016 John Darrington
+# Copyright (C) 2016, 2019 John Darrington
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,6 +15,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#purpose:  Test to make sure that things fail gracefully when the
+#purpose: stdin is not a tty but interactive mode is desired.
 
 tempdir=$(mktemp -d)
 
@@ -37,7 +39,7 @@ no_result ()
 }
 
 
-echo reset | $BINARY --no-bounce 2> $tempdir/errmsg
+echo reset | $BINARY --no-bounce --no-gdb-server 2> $tempdir/errmsg
 if test $? -eq 0; then fail; fi
 
 diff $tempdir/errmsg - <<EOF
