@@ -1,6 +1,6 @@
 /*
   upad - A program for debugging, and uploading code to embedded devices.
-  Copyright (C) 2016 John Darrington
+  Copyright (C) 2016, 2019 John Darrington
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -22,11 +22,18 @@
 #include <stdint.h>
 #include <stddef.h>
 
+#include "upad.h"
+
 struct upload_format
 {
-  bool (*get_data_fragment) (struct upload_format *uf, uint32_t *address,
+  bool (*get_data_fragment) (struct upload_format *uf, tgt_addr *address,
                              uint8_t *buffer, size_t bufsize, size_t *n_data);
   void (*destroy) (struct upload_format *uf);
 };
 
 #endif
+
+/* Local Variables:  */
+/* mode: c           */
+/* c-style: "gnu"    */
+/* End:              */
