@@ -1,6 +1,6 @@
 /*
   upad - A program for debugging, and uploading code to embedded devices.
-  Copyright (C) 2016 John Darrington
+  Copyright (C) 2016, 2019 John Darrington
 
   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include "user-interface.h"
 
+#include <assert.h>
 #include <stdio.h>
 #include <sys/stat.h>
 
@@ -30,15 +31,19 @@
 #include <signal.h>
 #include "lib/md5.h"
 #include "transaction.h"
+#include "options.h"
 #include "misc.h"
 #include "registers.h"
 #include "srecord-uploader.h"
 #include "binary-uploader.h"
+#include "elf-uploader.h"
 
 #if HAVE_WORDEXP_H
 # include <wordexp.h>
 #endif
 
+#include "gdb-interface.h"
+
 static const enum ARG_TYPE echo_args[] = {ARG_FILENAME, ARG_FILENAME};
 static const enum ARG_TYPE write_bdccsr_args[] = {ARG_DATUM};
 static const enum ARG_TYPE read_memory_args[] = {ARG_ADDRESS, ARG_QUANTITY};
@@ -54,7 +59,7 @@ start_upload_binary (int des, int argc, char **argv)
     }
 
   char *file = argv[0];
-  uint32_t address = strtol (argv[1], NULL, 0);
+  tgt_addr address = strtol (argv[1], NULL, 0);
 
 #if HAVE_WORDEXP_H
   wordexp_t exp;
@@ -125,6 +130,67 @@ start_upload_srec (int des, int argc, char **argv)
   return ret;
 }
 
+static int
+start_upload_elf (int des, int argc, char **argv)
+{
+  if (argc < 1)
+    {
+      fprintf (stderr, "Usage: upload-elf <file-name>\n");
+      return 1;
+    }
+
+  char *file = argv[0];
+#if HAVE_WORDEXP_H
+  wordexp_t exp;
+  if (0 != wordexp (file, &exp, WRDE_NOCMD))
+    {
+      upad_msg (MSG_ERROR, "Cannot expand string `%s'\n", file);
+      wordfree (&exp);
+      return 1;
+    }
+  file = exp.we_wordv[0];
+#endif
+  FILE *fp = fopen (file, "r");
+  if (fp == NULL)
+    {
+      char *emsg = strerror (errno);
+      upad_msg (MSG_ERROR, "Cannot open file `%s': %s\n", file, emsg);
+      return 1;
+    }
+#if HAVE_WORDEXP_H
+  wordfree (&exp);
+#endif
+  struct upload_format *uf = create_elf_uploader (fp);
+  int ret = upload_format (des, uf, 0);
+  free (uf);
+  if (fp)
+    fclose (fp);
+  return ret;
+}
+
+static int
+send_autoconnect (int des, int argc, char **argv)
+{
+  if (argc < 1)
+    {
+      upad_msg (MSG_INFORMATION,
+		"Usage: auto-connect [on|off]\n");
+      return 1;
+    }
+
+  if (0 == strcmp (argv[0], "on"))
+    execute_autoconnect (des, true);
+  else if (0 == strcmp (argv[0], "off"))
+    execute_autoconnect (des, false);
+  else
+    {
+      upad_msg (MSG_INFORMATION,
+		"Usage: auto-connect [on|off]");
+      return 1;
+    }
+  return 0;
+}
+
 static int
 send_write_regs (int des, int argc, char **argv)
 {
@@ -133,8 +199,8 @@ send_write_regs (int des, int argc, char **argv)
   if (argc < 1 || argc % 2 == 1)
     {
       upad_msg (MSG_INFORMATION,
-		   "Usage: write-regs <reg0-name> <reg0-value>  "
-		   "<reg1-name> <reg1-value>  ... <regN-name> <regN-value>\n");
+		"Usage: write-regs <reg0-name> <reg0-value>  "
+		"<reg1-name> <reg1-value>  ... <regN-name> <regN-value>\n");
       return 1;
     }
 
@@ -183,7 +249,7 @@ send_write_memory (int des, int argc, char **argv)
       upad_msg (MSG_INFORMATION, "Usage: write-mem <address> <N> <byte>\n");
       return 1;
     }
-  uint32_t address = strtol (argv[0], NULL, 0);
+  tgt_addr address = strtol (argv[0], NULL, 0);
   size_t quantity = strtol (argv[1], NULL, 0);
   uint8_t fill = strtol (argv[2], NULL, 0);
 
@@ -193,9 +259,13 @@ send_write_memory (int des, int argc, char **argv)
 
   const struct memsector *ms = get_memory_class (address);
 
-  if (0 > execute_write_memory (des, address, quantity, data + 1, ms))
+  if (0 > execute_write_memory_full (des, address, quantity, data + 1, ms))
+    {
+      free (data);
       return -1;
+    }
 
+  free (data);
   return 0;
 }
 
@@ -207,10 +277,11 @@ disassemble (int des, int argc, char **argv)
       upad_msg (MSG_INFORMATION, "Usage: disassemble <address> <n-bytes>\n");
       return 1;
     }
-  uint32_t address = strtol (argv[0], NULL, 0);
+  tgt_addr address = strtol (argv[0], NULL, 0);
   int quantity = strtol (argv[1], NULL, 0);
 
-  const uint8_t *rx_data = execute_read_memory (des, address, quantity);
+  uint8_t *rx_data = alloca (quantity);
+  execute_read_memory (des, address, quantity, rx_data);
 
   char cmd[256];
   char filename[30] = "/tmp/dis.XXXXXX";
@@ -362,7 +433,7 @@ send_erase_sector (int des, int argc, char **argv)
       upad_msg (MSG_INFORMATION, "Usage: erase-sector <address>\n");
       return 1;
     }
-  uint32_t address = strtol (argv[0], NULL, 0);
+  tgt_addr address = strtol (argv[0], NULL, 0);
 
   return execute_erase_sector (des, address);
 }
@@ -375,7 +446,7 @@ send_md5_check (int des, int argc, char **argv)
       upad_msg (MSG_INFORMATION, "Usage: md5-check <address> <n-bytes>\n");
       return 1;
     }
-  uint32_t address = strtol (argv[0], NULL, 0);
+  tgt_addr address = strtol (argv[0], NULL, 0);
   int quantity = strtol (argv[1], NULL, 0);
 
   uint8_t result[MD5_DIGEST_SIZE];
@@ -397,7 +468,7 @@ send_echo (int des, int argc, char **argv)
   if (argc < 2)
     {
       upad_msg (MSG_INFORMATION,
-		   "Usage: echo <infile> <outfile>\n");
+		"Usage: echo <infile> <outfile>\n");
       return 1;
     }
 
@@ -420,12 +491,12 @@ send_echo (int des, int argc, char **argv)
   if (fpout == NULL)
     {
       upad_errno (MSG_ERROR, "Cannot open file %s for writing",
-		     argv[1]);
+		  argv[1]);
       return 1;
     }
 
   if (0 != execute_echo (des, statbuf.st_size, fpin, fpout))
-      goto fail;
+    goto fail;
 
   fclose (fpout);
   return 0;
@@ -437,10 +508,10 @@ send_echo (int des, int argc, char **argv)
 
 
 static void
-print_data_as_numeric (uint32_t base, uint32_t offset, int quantity,
+print_data_as_numeric (tgt_addr base, tgt_addr offset, int quantity,
                        const uint8_t *data, FILE *stream)
 {
-  uint32_t a;
+  tgt_addr a;
   for (a = base ; a < base + 16; ++a)
     {
       if (a < base + offset)
@@ -455,10 +526,10 @@ print_data_as_numeric (uint32_t base, uint32_t offset, int quantity,
 }
 
 static void
-print_data_as_char (uint32_t base, uint32_t offset, int quantity,
+print_data_as_char (tgt_addr base, tgt_addr offset, int quantity,
                     const uint8_t *data, FILE *stream)
 {
-  uint32_t a;
+  tgt_addr a;
   for (a = base ; a < base + 16; ++a)
     {
       if (a < base + offset)
@@ -484,13 +555,14 @@ send_read_memory (int des, int argc, char **argv)
       upad_msg (MSG_INFORMATION, "Usage: read-mem <address> <n-bytes>\n");
       return 1;
     }
-  uint32_t address = strtol (argv[0], NULL, 0);
+  tgt_addr address = strtol (argv[0], NULL, 0);
   int quantity = strtol (argv[1], NULL, 0);
 
-  const uint8_t *rx_data = execute_read_memory (des, address, quantity);
+  uint8_t *rx_data = alloca (quantity);
+  execute_read_memory (des, address, quantity, rx_data);
 
-  uint32_t offset = address % 16;
-  uint32_t base = address - offset;
+  tgt_addr offset = address % 16;
+  tgt_addr base = address - offset;
 
   while (quantity > 0)
     {
@@ -535,6 +607,9 @@ static const char lack_of_warranty[] =
   "ALL NECESSARY SERVICING, REPAIR OR CORRECTION.\n";
 
 
+
+/* Show auxialliary information.   Currently supports
+   two options - "warranty" and "copying".  */
 static int
 show (int des, int argc, char **argv)
 {
@@ -762,6 +837,20 @@ const struct xcmd cmds[] =
       prior to calling this command.
     */
     {"upload-srecord", start_upload_srec,   1, NULL},
+    /*-
+      @node upload-elf
+      @section upload-elf
+
+      @cindex elf, file format
+      The @command{upload-elf} command reads a file from the local computer
+      and uploads the appropriate sections to the target device.    The command takes a single argument
+      which is the name of the file to upload.
+
+      This command will not erase any memory before uploading.   So it will
+      probably fail if you do not issue a @command{bulk-erase} command (@pxref{bulk-erase})
+      prior to calling this command.
+    */
+    {"upload-elf", start_upload_elf,   1, NULL},
     /*-
       @node upload-binary
       @section upload-binary
@@ -800,13 +889,24 @@ const struct xcmd cmds[] =
       be aligned to a sector boundary; anywhere within the sector will suffice.
     */
     {"erase-sector", send_erase_sector,     1, NULL},
-  };
+    /*-
+      @node auto-connect
+      @section auto-connect
+
+      @cindex Auto Connect
+      The @command{auto-connect} enabled or disabled auto-connection.
+      The command takes a single argument: either @samp{on} or @samp{off}
+    */
+    {"auto-connect",  send_autoconnect,      1, NULL},
+};
 
 
 static char *
 command_generator (const char *text, int state)
 {
   char *delim = strchr (rl_line_buffer, ' ');
+
+  /* Special case these two because they contain hyphens.   */
   if (delim &&
       (0 == strncmp (rl_line_buffer, "upload-srecord", delim - rl_line_buffer)
        ||
@@ -842,9 +942,9 @@ initialise_history (void)
 {
   char *home = getenv ("HOME");
   char *histfile = NULL;
-  char histfilename[] = ".upad_history";
   if (home)
     {
+      char histfilename[] = ".upad_history";
       histfile = safe_realloc (NULL, strlen (home) + strlen (histfilename) + 2);
 
       if (histfile)
@@ -857,7 +957,7 @@ initialise_history (void)
 
 	  int rh = read_history (histfile);
 	  if (rh != 0 && rh != ENOENT)
-	    perror ("Cannot read history");
+	    upad_errno (MSG_ERROR, "Cannot read history");
 	}
     }
   return histfile;
@@ -872,17 +972,78 @@ static const char splash[] =
   "under certain conditions; type `show copying' for details.\n";
 
 
-int
-run_interactive (int des)
+static struct user_interface *global_ui = NULL;
+
+/* Readline handler for the user interface.  */
+static void
+cb_linehandler (char *s)
+{
+  if (s == NULL)
+    {
+      global_ui->running = false;
+      rl_callback_handler_remove ();
+      return;
+    }
+
+  if (strlen (s) == 0)
+    goto done;
+
+  char *expansion = NULL;
+  int result = history_expand (s, &expansion);
+  if (result < 0)
+    {
+      free (expansion);
+      upad_msg (MSG_ERROR, "Event not found\n");
+      goto done;
+    }
+  if (result > 0)
+    upad_msg (MSG_ERROR, "%s\n", expansion);
+
+  if (global_ui->cmd_length < strlen (expansion))
+    {
+      global_ui->cmd_length = strlen (expansion);
+      global_ui->cmd =
+        safe_realloc (global_ui->cmd, global_ui->cmd_length + 1);
+    }
+  strcpy (global_ui->cmd, expansion);
+  free (expansion);
+
+  if (global_ui->xxdes >= 0 && !process_command (global_ui->cmd,
+                                                 global_ui->xxdes))
+    {
+      upad_msg (MSG_ERROR, "Bad command\n");
+    }
+
+  {
+    int where = where_history ();
+    HIST_ENTRY *he = history_get (where);
+    if (he && (0 == strcmp (he->line, global_ui->cmd)))
+      goto done;
+    add_history (global_ui->cmd);
+  }
+ done:
+  free (s);
+}
+
+
+/* Create a user interface and return a struct representing
+   it.  */
+struct user_interface *
+user_interface_init (const struct user_options *opts)
 {
   if (! isatty (fileno (stdin)))
     {
       fprintf (stderr, "Interactive mode is only possible from a terminal.\n");
-      return 1;
+      return NULL;
     }
 
+  struct user_interface  *ui = safe_realloc (NULL, sizeof (*ui));
+  memset (ui, 0, sizeof (*ui));
+  ui->xxdes = -1;
+
+  if (!opts->no_splash)
   /* Interactive mode */
-  fputs (splash, stdout);
+    fputs (splash, stdout);
 
   rl_prep_terminal (0);
 
@@ -890,71 +1051,63 @@ run_interactive (int des)
 
   sa.sa_handler = int_handler;
   sigemptyset (&sa.sa_mask);
-  sa.sa_flags = SA_RESTART; /* Restart functions if interrupted by handler */
+  /* Restart functions if interrupted by handler */
+  sa.sa_flags = SA_RESTART;
+
+  assert (sizeof (cmds) / sizeof (cmds[0]) == N_COMMANDS);
+
   if (sigaction (SIGINT, &sa, NULL) == -1)
-    perror ("Cannot set handler for SIGINT");
+    upad_errno (MSG_ERROR, "Cannot set handler for SIGINT");
 
   rl_completion_entry_function = command_generator;
 
-  char *histfile = initialise_history ();
+  ui->histfile = initialise_history ();
 
-  int cmd_length = 0;
-  char *cmd = NULL;
-  for (;;)
-    {
-      char *s = readline ("> ");
-      if (!s)
-	break;
+  rl_callback_handler_install ("> ", cb_linehandler);
+  ui->ui_des = fileno (rl_instream);
 
-      if (strlen (s) == 0)
-	goto done;
+  ui->running = true;
 
-      char *expansion = NULL;
-      int result = history_expand (s, &expansion);
-      if (result < 0)
-	{
-	  free (expansion);
-	  upad_msg (MSG_ERROR, "Event not found\n");
-	  goto done;
-	}
-      if (result > 0)
-	upad_msg (MSG_ERROR, "%s\n", expansion);
-
-      if (cmd_length < strlen (expansion))
-	{
-          cmd_length = strlen (expansion);
-          cmd = safe_realloc (cmd, cmd_length + 1);
-	}
-      strcpy (cmd, expansion);
-      free (expansion);
-
-      if (!process_command (cmd, cmd_length, des))
-	{
-	  upad_msg (MSG_ERROR, "Bad command\n");
-	}
-
-      {
-	int where = where_history ();
-	HIST_ENTRY *he = history_get (where);
-	if (he && (0 == strcmp (he->line, cmd)))
-	  goto done;
-	add_history (cmd);
-      }
-    done:
-      free (s);
-    }
-  free (cmd);
-
-  if (histfile)
-    {
-      int rh = write_history (histfile);
-      if (rh != 0)
-	{
-	  perror ("Cannot write history");
-	}
-    }
-
-  free (histfile);
-
-  return 0;
+  return ui;
 }
+
+
+/* Close the user interface and free its resources.  */
+void
+user_interface_shutdown (struct user_interface *ui)
+{
+  if (ui == NULL)
+    return;
+
+  ui->running = false;
+  global_ui = NULL;
+  rl_callback_handler_remove ();
+
+  if (ui->histfile)
+    {
+      int rh = write_history (ui->histfile);
+      if (rh != 0)
+        {
+          upad_errno (MSG_ERROR, "Cannot write %s (history)", ui->histfile);
+        }
+    }
+
+  free (ui->histfile);
+  free (ui->cmd);
+
+  free (ui);
+}
+
+/* Service the user interface.  This is called once in every iteraction
+   of the main loop.  */
+void
+user_interface_process (struct user_interface *ui)
+{
+  global_ui = ui;
+  rl_callback_read_char ();
+}
+
+/* Local Variables:  */
+/* mode: c           */
+/* c-style: "gnu"    */
+/* End:              */
